package explicit;

import java.util.BitSet;
import java.util.Vector;

import jltl2ba.APElement;

import prism.PrismException;

import jltl2dstar.DA_State;
import jltl2dstar.DRA;

public class DRAProductOperator<G> implements ProductOperator<G>
{
	private G graph;	
	private DRA dra;
	Vector<BitSet> labeled_states;
	private RabinAcceptance acceptance = new RabinAcceptance();

	
	public DRAProductOperator(G graph, DRA dra, Vector<BitSet> labeled_states) throws PrismException
	{
		this.graph = graph;
		this.dra = dra;
		this.labeled_states = labeled_states;
		
		for (int i=0;i<dra.acceptance().size();i++) {
			acceptance.addPair(new RabinPair());
		}
	}

	
	public RabinAcceptance getRabinAcceptance()
	{
		return acceptance;
	}
	
	@Override
	public ProductState getInitialState(Integer graph_state)
	{
		APElement labels = getLabelsForState(graph_state);
		Integer dra_state = getDRASuccessor(dra.getStartState().getName(), labels);
		return new ProductState(graph_state, dra_state);
	}

	
	@Override
	public ProductState getSuccessor(ProductState from_state, Integer graph_to_state)
	{
		APElement labels = getLabelsForState(graph_to_state);
		Integer dra_to_state = getDRASuccessor(from_state.getSecondState(), labels);
		return new ProductState(graph_to_state, dra_to_state);
	}

	@Override
	public void notify(ProductState product_state, Integer product_index) throws PrismException
	{
		Integer dra_state = product_state.getSecondState();
		// calculate Rabin acceptance of the product state
		for (int pair = 0; pair<dra.acceptance().size(); pair++) {
			// <>[]!U &&  []<>L
			boolean in_ltl2dstar_L = dra.acceptance().isStateInAcceptance_L(pair, dra_state);
			boolean in_ltl2dstar_U = dra.acceptance().isStateInAcceptance_U(pair, dra_state);

			boolean in_H = !in_ltl2dstar_U;
			boolean in_L = in_ltl2dstar_L;
			
			acceptance.getPair(pair).getH().set(product_index, in_H);
			acceptance.getPair(pair).getL().set(product_index, in_L);
		}
	}

	
	@Override
	public G getGraph()
	{
		return graph;
	}

	@Override
	public Object getExtraInfo()
	{
		return this;
	}

	private Integer getDRASuccessor(Integer dra_from, APElement labels) {
		DA_State da_state = dra.get(dra_from.intValue()).edges().get(labels);
		return da_state.getName();
	}
	
	private APElement getLabelsForState(Integer graph_state) {
		APElement labels = new APElement();
		int ap_index_dra = 0;
		for (String ap : dra.getAPSet() ) {
			// ap is of the form L0, L1, ....
			int ap_index_dtmc = Integer.valueOf(ap.substring(1));
			if (labeled_states.get(ap_index_dtmc).get(graph_state.intValue())) {
				labels.set(ap_index_dra);
			}
			
			ap_index_dra++;
		}
		return labels;
	}
	
}
