package explicit;

import java.util.BitSet;
import java.util.Vector;

import parser.ast.Expression;
import prism.LTL2RabinLibrary;
import prism.Prism;
import prism.PrismException;
import prism.PrismLog;
import jltl2ba.SimpleLTL;
import jltl2dstar.DRA;
import jltl2dstar.LTL2Rabin;

public class LTLProduct<P>
{
	
	public P getProductGraph()
	{
		return product_graph;
	}
	
	public RabinAcceptance getAcceptance()
	{
		return acceptance;
	}
	
	
	public static LTLProduct<DTMCProduct> getProduct(Prism prism, DTMC dtmc, Expression ltl, Vector<BitSet> labeledStates, boolean allInit) throws PrismException
	{
		PrismLog log = prism.getMainLog();
		
		LTL2RabinLibrary.checkIsSupportedLTL(ltl);
		
		// Convert LTL formula to deterministic Rabin automaton (DRA)
		log.println("\nBuilding deterministic Rabin automaton (for " + ltl + ")...");
		long l = System.currentTimeMillis();

		DRA dra = LTL2RabinLibrary.getDRAForLTLFormula(ltl);
		
		log.println("\nDRA has " + dra.size() + " states, " + dra.acceptance().size() + " pairs.");
		// dra.print(System.out);
		l = System.currentTimeMillis() - l;
		log.println("\nTime for Rabin translation: " + l / 1000.0 + " seconds.");

		log.println("\nConstructing MC-DRA product...");

		
		DRAProductOperator<DTMC> product_operator
			= new DRAProductOperator<DTMC>(dtmc, dra, labeledStates);

		DTMCProduct product = DTMCProduct.generate(product_operator, allInit);
	
		log.println();
		log.print("States:      " + product.getNumStates() + " (" + product.getNumInitialStates() + " initial)" + "\n");
		log.print("Transitions: " + product.getNumTransitions() + "\n");

//		dtmc.exportToDotFile("dtmc.dot");
//		product.exportToDotFile("product.dot");

		RabinAcceptance acceptance = product_operator.getRabinAcceptance();
//		log.println(acceptance);

				
		// Output product, if required
		if (prism.getExportProductTrans()) {
			log.println("\nExporting product transition matrix to file \"" + prism.getExportProductTransFilename() + "\"...");
			product.exportToDotFile(prism.getExportProductTransFilename(), null, acceptance);
		}
//      if (prism.getExportProductStates()) {
//			log.println("\nExporting product state space to file \"" + prism.getExportProductStatesFilename() + "\"...");
//			product.exportStates(Prism.EXPORT_PLAIN, new PrismFileLog(prism.getExportProductStatesFilename()));
//		}

		
		LTLProduct<DTMCProduct> result = new LTLProduct<DTMCProduct>();
		result.product_graph = product;
		result.acceptance = acceptance;
		return result;
	}
	
	private P product_graph;
	private RabinAcceptance acceptance;
}
