package explicit;

import java.util.ArrayList;
import java.util.List;

/** Disjunction over the RabinPairs */
public class RabinAcceptance
{
	private List<RabinPair> pairs = new ArrayList<RabinPair>();
	
	int countPairs()
	{
		return pairs.size();
	}
	
	int addPair(RabinPair pair)
	{
		int index = pairs.size();
		pairs.add(pair);
		return index;
	}
	
	RabinPair getPair(int index)
	{
		return pairs.get(index);
	}
	
	Iterable<RabinPair> getPairs()
	{
		return pairs;
	}

	public String toString() {
		String result = "";
		for (RabinPair pair : pairs) {
			result += pair+ " ";
		}
		return result;
	}
	
	/**
	 * Get the Rabin signature for the given state.
	 * -i if the state is not allowed to occur infinitely often in pair i.
	 * +i if the state is contained in the recurrence set of pair i.
	 * @param state
	 * @return
	 */
	public String getSignature(int state) {
		String signature = "";
		for (int i=0;i<countPairs();i++) {
			if (getPair(i).getH().get(state) == false) {
				if (!signature.isEmpty()) signature += " ";
				signature += "-"+i;
			} else if (getPair(i).getL().get(state) == true) {
				if (!signature.isEmpty()) signature += " ";
				signature += "+"+i;
			}
		}
		return signature;
	}
}
