package explicit.conditional;

import java.io.FileWriter;
import java.io.IOException;
import java.util.BitSet;
import java.util.Iterator;
import java.util.Map;

import prism.PrismException;
import explicit.DTMC;
import explicit.MDP;

public class ModelPrinter {

	public static void exportToDotFile(DTMC model, String filename, BitSet mark) throws PrismException {
		FileWriter out = null;
		try {
			out = new FileWriter(filename);
			out.write("digraph " + model.getModelType() + " {\nsize=\"8,5\"\nnode [shape=box];\n");
			for (int i = 0; i < model.getNumStates(); i++) {
				String style = "style=\"";
				String fillcolor = "";
				if(model.isInitialState(i)) {
					style += "dashed";
				} else {
					style += "solid";
				}
				if (mark != null && mark.get(i)) {
					style += ",filled";
					fillcolor = "fillcolor=\"#cccccc\"";
				}
				out.write(i + " [" + style + "\" " + fillcolor +"]\n");
				Iterator<Map.Entry<Integer, Double>> iter = model.getTransitionsIterator(i);
				while (iter.hasNext()) {
					Map.Entry<Integer, Double> e = iter.next();
					out.write(i + " -> " + e.getKey() + " [ label=\"");
					out.write(e.getValue() + "\" ];\n");
				}
			}
			out.write("}\n");
		} catch (IOException e) {
			throw new PrismException("Could not write " + model.getModelType() + " to file \"" + filename + "\"" + e);
		} finally {
			if (out != null) {
				try {
					out.close();
				} catch (IOException e) {
					// Done
				}
			}
		}
	}

	public static void exportToDotFile(ModelTransformation transformation, final String filename, final BitSet mark) throws PrismException {
		DTMC model = transformation.getTransformedModel();
		FileWriter out = null;
		try {
			out = new FileWriter(filename);
			out.write("digraph " + model.getModelType() + " {\nsize=\"8,5\"\nnode [shape=box];\n");
			out.write("labelloc=\"b\";\n");
			out.write("label=\"" + transformation.getClass().getSimpleName() + "\";\n");
			for (int i = 0; i < model.getNumStates(); i++) {
				out.write(i + transformation.getStateAttributes(i, mark) + ";\n");
				Iterator<Map.Entry<Integer, Double>> iter = model.getTransitionsIterator(i);
				while (iter.hasNext()) {
					Map.Entry<Integer, Double> e = iter.next();
					out.write(i + " -> " + e.getKey() + " [ label=\"");
					out.write(e.getValue() + "\" ];\n");
				}
			}
			out.write("}\n");
			out.close();
		} catch (IOException e) {
			throw new PrismException("Could not write " + model.getModelType() + " to file \"" + filename + "\"" + e);
		} finally {
			if (out != null) {
				try {
					out.close();
				} catch (IOException e) {
					// Done
				}
			}
		}
	}

	public static void exportToDotFile(MDP model, String filename, BitSet mark) throws PrismException {
		FileWriter out = null;
		try {
			out = new FileWriter(filename);
			out.write("digraph " + model.getModelType() + " {\nsize=\"8,5\"\nnode [shape=box];\n");
			for (int i = 0; i < model.getNumStates(); i++) {
				String style = "style=\"";
				String fillcolor = "";
				if(model.isInitialState(i)) {
					style += "dashed";
				} else {
					style += "solid";
				}
				if (mark != null && mark.get(i)) {
					style += ",filled";
					fillcolor = "fillcolor=\"#cccccc\"";
				}
				out.write(i + " [" + style + "\" " + fillcolor +"]\n");
				int numChoices = model.getNumChoices(i);
				for (int j = 0; j < numChoices; j++) {
					Object action = model.getAction(i, j);
					String nij = "n" + i + "_" + j;
					out.write(i + " -> " + nij + " [ arrowhead=none,label=\"" + j);
					if (action != null) {
						out.write(":" + action);
					}
					out.write("\" ];\n");
					out.write(nij + " [ shape=point,width=0.1,height=0.1,label=\"\" ];\n");
					Iterator<Map.Entry<Integer, Double>> iter = model.getTransitionsIterator(i, j);
					while (iter.hasNext()) {
						Map.Entry<Integer, Double> e = iter.next();
						out.write(nij + " -> " + e.getKey() + " [ label=\"" + e.getValue() + "\" ];\n");
					}
				}
			}
			out.write("}\n");
		} catch (IOException e) {
			throw new PrismException("Could not write " + model.getModelType() + " to file \"" + filename + "\"" + e);
		} finally {
			if (out != null) {
				try {
					out.close();
				} catch (IOException e) {
					// Done
				}
			}
		}
	}

	public static void exportToDotFile(ModelTransformation transformation, MDP model, final String filename, final BitSet mark) throws PrismException {
		FileWriter out = null;
		try {
			out = new FileWriter(filename);
			out.write("digraph " + model.getModelType() + " {\nsize=\"8,5\"\nnode [shape=box];\n");
			out.write("labelloc=\"b\";\n");
			out.write("label=\"" + transformation.getClass().getSimpleName() + "\";\n");
			for (int i = 0; i < model.getNumStates(); i++) {
				out.write(i + transformation.getStateAttributes(i, mark) + ";\n");
				int numChoices = model.getNumChoices(i);
				for (int j = 0; j < numChoices; j++) {
					Object action = model.getAction(i, j);
					String nij = "n" + i + "_" + j;
					out.write(i + " -> " + nij + " [ arrowhead=none,label=\"" + j);
					if (action != null) {
						out.write(":" + action);
					}
					out.write("\" ];\n");
					out.write(nij + " [ shape=point,width=0.1,height=0.1,label=\"\" ];\n");
					Iterator<Map.Entry<Integer, Double>> iter = model.getTransitionsIterator(i, j);
					while (iter.hasNext()) {
						Map.Entry<Integer, Double> e = iter.next();
						out.write(nij + " -> " + e.getKey() + " [ label=\"" + e.getValue() + "\" ];\n");
					}
				}
			}
			out.write("}\n");
		} catch (IOException e) {
			throw new PrismException("Could not write " + model.getModelType() + " to file \"" + filename + "\"" + e);
		} finally {
			if (out != null) {
				try {
					out.close();
				} catch (IOException e) {
					// Done
				}
			}
		}
	}
}
