/**
 * 
 */
package parser.ast;

import parser.EvaluateContext;
import parser.visitor.ASTVisitor;
import prism.PrismLangException;

/**
 * @author Steffen
 *
 */
public class ExpressionConditional extends Expression {

	private Expression expr;
	private Expression cond;

	public ExpressionConditional(Expression expr, Expression cond) {
		assert (expr != null) && (cond != null)
				: "expression and condition must not be null";;
		this.expr = expr;
		this.cond = cond;
	}

	@Deprecated
	public Expression getExpr() {
		return expr;
	}

	@Deprecated
	public Expression getCond() {
		return cond;
	}

	/* (non-Javadoc)
	 * @see parser.ast.Expression#isConstant()
	 */
	@Override
	public boolean isConstant() {
		return false;
	}

	public boolean isQuantiative() {
		// TODO check how this attribute is processed
		return (expr == null) ? false : Expression.isQuantitative(expr);
	}

	/* (non-Javadoc)
	 * @see parser.ast.Expression#evaluate(parser.EvaluateContext)
	 */
	@Override
	public Object evaluate(EvaluateContext ec) throws PrismLangException {
		throw new PrismLangException("Cannot evaluate a conditional expression without a model");
	}

	/* (non-Javadoc)
	 * @see parser.ast.Expression#returnsSingleValue()
	 */
	@Override
	public boolean returnsSingleValue() {
		return false;
	}

	/* (non-Javadoc)
	 * @see parser.ast.Expression#deepCopy()
	 */
	@Override
	public Expression deepCopy() {
		Expression exprCopy = (expr == null) ? null : expr.deepCopy();
		Expression condCopy = (cond == null) ? null : cond.deepCopy();
		ExpressionConditional copy = new ExpressionConditional(exprCopy, condCopy);
		copy.setPosition(this);
		copy.setType(type);
		return copy;
	}

	/* (non-Javadoc)
	 * @see parser.ast.ASTElement#accept(parser.visitor.ASTVisitor)
	 */
	@Override
	public Object accept(ASTVisitor v) throws PrismLangException {
		return v.visit(this);
	}

	/* (non-Javadoc)
	 * @see parser.ast.ASTElement#toString()
	 */
	@Override
	public String toString() {
		return expr + "[ " + cond + " ]";
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((cond == null) ? 0 : cond.hashCode());
		result = prime * result + ((expr == null) ? 0 : expr.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (!(obj instanceof Expression)) {
			return false;
		}
		Expression trimmed = ((Expression) obj).trimUnaryOperations();
		if (!(trimmed instanceof ExpressionConditional)) {
			return false;
		}
		ExpressionConditional other = (ExpressionConditional) trimmed;

		if (cond == null) {
			if (other.cond != null) {
				return false;
			}
		} else if (!cond.equals(other.cond)) {
			return false;
		}
		if (expr == null) {
			if (other.expr != null) {
				return false;
			}
		} else if (!expr.equals(other.expr)) {
			return false;
		}
		return true;
	}
}
