package prism;

public class IntegerBound
{
	public IntegerBound(Integer lower, boolean lower_strict, Integer upper, boolean upper_strict) {
		// normalize
		if (lower_strict && lower != null) {
			lowest = new Integer(lower+1);
		} else {
			lowest = lower;
		}
		if (upper_strict && upper != null) {
			highest = new Integer(upper-1);
		} else {
			highest = upper;
		}
	}
	
	public boolean hasNegativeBounds() {
		if (lowest != null && lowest.intValue() < 0 ||
		    highest != null && highest.intValue() < 0) {
			return true;
		}
		return false;
	}
	
	/** Returns true if value is in bound. */
	public boolean isInBound(int value) {
		if (lowest!=null) {
			if (value >= lowest.intValue()) {
				// continue
			} else {
				return false;
			}
		}
		
		if (highest!=null) {
			if (value <= highest.intValue()) {
				return true;
			} else {
				return false;
			}
		}
		
		// no bound (upper=lower=null)
		return true;
	}
	
	/** Get the maximal interesting value, i.e., the value v such that
	 *  for _all_ i>=v either isInBound(i)=true or isInBound(i)=false */
	public int getMaximalInterestingValue() {
		int max = 0;
		if (lowest != null) {
			max = lowest.intValue();
		}
		if (highest != null) {
			int value = highest.intValue()+1;
			if (value > max)
				max = value;
		}
		return max;
	}
	
	private Integer lowest, highest;
}
